@@ -155,8 +155,18 @@ class Cache:
                 cache_file_path = cache_dir_path + str(func.__name__) + '.ff1pkl'
                 if os.path.isfile(cache_file_path):
                     # file exists already, try to load it
-                    cached = pickle.load(open(cache_file_path, 'rb'))
-                    if ((cached['version'] == cls._API_CORE_VERSION) or cls._IGNORE_VERSION) and not cls._FORCE_RENEW:
+                    try:
+                        cached = pickle.load(open(cache_file_path, 'rb'))
+                    except:  # noqa: E722 (bare except)
+                        # don't like the bare exception clause but who knows
+                        # which dependency will raise which internal exception
+                        # after it was updated
+                        cached = None
+
+                    if (cached is not None and
+                            ((cached['version'] == cls._API_CORE_VERSION) or
+                             cls._IGNORE_VERSION)
+                            and not cls._FORCE_RENEW):
                         # was created with same version or version is ignored
                         logging.info(f"Using cached data for {str(func.__name__)}")
                         return cached['data']
@@ -1039,6 +1049,76 @@ def session_status_data(path, response=None, livedata=None):
     return data
 
 
+@Cache.api_request_wrapper
+def weather_data(path, response=None, livedata=None):
+    """Fetch and parse weather data.
+
+    Weather data provides the following data channels per sample:
+
+        - Time (datetime.timedelta): session timestamp (time only)
+        - AirTemp (float): Air temperature [°C]
+        - Humidity (float): Relative humidity [%]
+        - Pressure (float): Air pressure [mbar]
+        - Rainfall (bool): Shows if there is rainfall
+        - TrackTemp (float): Track temperature [°C]
+        - WindDirection (int): Wind direction [°] (0°-359°)
+        - WindSpeed (float): Wind speed [km/h]
+
+    Weather data is updated once per minute.
+
+    Args:
+        path (str): api path base string (see :func:`api.make_path`)
+        response: Response as returned by :func:`api.fetch_page` can
+            be passed if it was downloaded already.
+        livedata: An instance of
+            :class:`fastf1.livetiming.data.LivetimingData`
+            to use as a source instead of the api
+
+    Returns:
+        A dictionary containing one key for each data channel and a list
+        of values per key.
+
+    Raises:
+        SessionNotAvailableError: in case the F1 live timing api
+            returns no data
+    """
+    if livedata is not None and livedata.has('WeatherData'):
+        # does not need any further processing
+        logging.info("Loading weather data")
+        response = livedata.get('WeatherData')
+    elif response is None:
+        logging.info("Fetching weather data...")
+        response = fetch_page(path, 'weather_data')
+        if response is None:  # no response received
+            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+
+    data = {
+        'Time': [], 'AirTemp': [], 'Humidity': [], 'Pressure': [],
+        'Rainfall': [], 'TrackTemp': [], 'WindDirection': [], 'WindSpeed': []
+    }
+
+    data_keys = ('AirTemp', 'Humidity', 'Pressure', 'Rainfall',
+                 'TrackTemp', 'WindDirection', 'WindSpeed')
+    data_dtypes = (float, float, float, bool, float, int, float)
+
+    for entry in response:
+        if len(entry) < 2:
+            continue
+        row = entry[1]
+        if not isinstance(row, dict):
+            continue
+
+        data['Time'].append(to_timedelta(entry[0]))
+        for key, dtype in zip(data_keys, data_dtypes):
+            try:
+                data[key].append(dtype(row[key]))
+            except (KeyError, ValueError):
+                # type conversion failed or key is missing
+                data[key].append(dtype(0))
+
+    return data
+
+
 def fetch_page(path, name):
     """Fetch data from the formula1 livetiming web api, given url base path and page name. An attempt
     to parse json or decode known messages is made.
