@@ -705,12 +705,17 @@ class Telemetry(pd.DataFrame):
         Returns:
             :class:`Telemetry`: self joined with new column or self if column exists and `drop_existing` is False.
         """
-        if 'Distance' in self.columns:
-            if drop_existing:
-                return self.drop('DifferentialDistance', 1)\
-                    .join(pd.DataFrame({'DifferentialDistance': self.calculate_differential_distance()}), how='outer')
+        if ('DifferentialDistance' in self.columns) and not drop_existing:
             return self
-        return self.join(pd.DataFrame({'DifferentialDistance': self.calculate_differential_distance()}), how='outer')
+
+        new_dif_dist = pd.DataFrame(
+            {'DifferentialDistance': self.calculate_differential_distance()}
+        )
+        if 'DifferentialDistance' in self.columns:
+            return self.drop(labels='DifferentialDistance', axis=1)\
+                .join(new_dif_dist, how='outer')
+
+        return self.join(new_dif_dist, how='outer')
 
     def add_distance(self, drop_existing=True):
         """Add column 'Distance' to self.
@@ -728,11 +733,14 @@ class Telemetry(pd.DataFrame):
         Returns:
             :class:`Telemetry`: self joined with new column or self if column exists and `drop_existing` is False.
         """
-        if 'Distance' in self.columns:
-            if drop_existing:
-                return self.drop('Distance', 1).join(pd.DataFrame({'Distance': self.integrate_distance()}), how='outer')
+        if ('Distance' in self.columns) and not drop_existing:
             return self
-        return self.join(pd.DataFrame({'Distance': self.integrate_distance()}), how='outer')
+
+        new_dist = pd.DataFrame({'Distance': self.integrate_distance()})
+        if 'Distance' in self.columns:
+            return self.drop(labels='Distance', axis=1).join(new_dist, how='outer')
+
+        return self.join(new_dist, how='outer')
 
     def add_relative_distance(self, drop_existing=True):
         """Add column 'RelativeDistance' to self.
@@ -750,7 +758,7 @@ class Telemetry(pd.DataFrame):
         """
         if 'RelativeDistance' in self.columns:
             if drop_existing:
-                d = self.drop('RelativeDistance', 1)
+                d = self.drop(labels='RelativeDistance', axis=1)
             else:
                 return self
         else:
@@ -760,7 +768,7 @@ class Telemetry(pd.DataFrame):
             rel_dist = d.loc[:, 'Distance'] / d.loc[:, 'Distance'].iloc[-1]
         else:
             dist = d.integrate_distance()
-            rel_dist = dist / dist.iloc[0]
+            rel_dist = dist / dist.iloc[-1]
         return d.join(pd.DataFrame({'RelativeDistance': rel_dist}), how='outer')
 
     def add_driver_ahead(self, drop_existing=True):
@@ -788,14 +796,17 @@ class Telemetry(pd.DataFrame):
         """
         if 'DriverAhead' in self.columns and 'DistanceToDriverAhead' in self.columns:
             if drop_existing:
-                d = self.drop('DriverAhead', 1).drop('DistanceToDriverAhead', 1)
+                d = self.drop(labels='DriverAhead', axis=1)\
+                    .drop(labels='DistanceToDriverAhead', axis=1)
             else:
                 return self
         else:
             d = self
 
         drv_ahead, dist = self.calculate_driver_ahead()
-        return d.join(pd.DataFrame({'DriverAhead': drv_ahead, 'DistanceToDriverAhead': dist}), how='outer')
+        return d.join(pd.DataFrame({'DriverAhead': drv_ahead,
+                                    'DistanceToDriverAhead': dist},
+                                   index=d.index), how='outer')
 
     def calculate_differential_distance(self):
         """Calculate the distance between subsequent samples of self.
@@ -1018,8 +1029,11 @@ class Session:
     """
     def __init__(self, weekend, session_name):
         self.weekend = weekend
+        """:class:`.Weekend`: Reference to the associated weekend object."""
         self.name = session_name
+        """str: Name of this session, for example 'Qualifying', 'Race', 'FP1', ..."""
         self.date = self._get_session_date()
+        """pandas.Datetime: Date at which this session took place."""
         self.api_path = api.make_path(self.weekend.name,
                                       self.weekend.date,
                                       self.name, self.date)
@@ -1045,11 +1059,14 @@ class Session:
             self._create_empty_ergast_result()
 
         self.laps = Laps(session=self)
-        """Instance of :class:`Laps` containing all laps from all drivers in this session."""
-        self.t0_date = None  # can only be set when/if telemetry has been downloaded
-        """Date timestamp which marks the beginning of the data stream."""
+        """Instance of :class:`Laps` containing all laps from all drivers
+        in this session."""
+        self.t0_date = None
+        # can only be set when/if telemetry has been downloaded
+        """pandas.Datetime: Date timestamp which marks the beginning of the
+        data stream."""
         self.session_start_time = None
-        """Session time at which the session was started."""
+        """pandas.Timedelta: Session time at which the session was started."""
 
         self.car_data = dict()
         """Dictionary of car telemetry (Speed, RPM, etc.) as received from
@@ -1059,6 +1076,10 @@ class Session:
         """Dictionary of car position data as received from the api by car
         number (where car number is a string and the telemetry
         is an instance of :class:`Telemetry`)"""
+        self.weather_data = pd.DataFrame()
+        """Dataframe containing weather data for this session as received
+        from the api. See :func:`fastf1.api.weather_data` for available data
+        channels. Each data channel is one row of the dataframe."""
 
         self.drivers = list()
         """List of all drivers that took part in this session; contains driver numbers as string. Drivers for which
@@ -1246,6 +1267,13 @@ class Session:
         if with_telemetry:
             self.load_telemetry(livedata=livedata)
 
+        # load weather data; this data is not crucial, just log a warning
+        # in case of an error
+        try:
+            self._load_weather_data(livedata=livedata)
+        except (api.SessionNotAvailableError, ValueError):
+            logging.warning("Failed to load weather data!")
+
         logging.info(f"Loaded data for {len(self.drivers)} drivers: {self.drivers}")
 
         return self.laps
@@ -1296,8 +1324,19 @@ class Session:
             if integrity_errors > 0:
                 logging.warning(f"Driver {drv: >2}: Lap timing integrity check failed for {integrity_errors} lap(s)")
 
+    def _load_weather_data(self, livedata=None):
+        """Get weather data from the api and format it into a dataframe"""
+        weather_data = api.weather_data(self.api_path, livedata=livedata)
+        weather_df = pd.DataFrame(weather_data)
+        self.weather_data = weather_df
+
     def load_telemetry(self, livedata=None):
-        """Load telemetry data from API.
+        """Load telemetry data from the API.
+
+        This method can only be called after :meth:`load_laps` has been
+        called. You will usually just want to call :meth:`load_laps` with
+        the optional ``with_telemetry=True`` argument instead of calling this
+        method separately. The result will be the same.
 
         The raw data is divided into car data (Speed, RPM, ...) and position data (coordinates, on/off track). For each
         of the two types an instance of :class:`Telemetry` is created per driver. The properties
@@ -1326,8 +1365,10 @@ class Session:
 
         for drv in self.drivers:
             # drop and recalculate time stamps based on 'Date', because 'Date' has a higher resolution
-            drv_car = Telemetry(car_data[drv].drop('Time', 1), session=self, driver=drv)
-            drv_pos = Telemetry(pos_data[drv].drop('Time', 1), session=self, driver=drv)
+            drv_car = Telemetry(car_data[drv].drop(labels='Time', axis=1),
+                                session=self, driver=drv)
+            drv_pos = Telemetry(pos_data[drv].drop(labels='Time', axis=1),
+                                session=self, driver=drv)
 
             drv_car['Date'] = drv_car['Date'].round('ms')
             drv_pos['Date'] = drv_pos['Date'].round('ms')
@@ -1416,7 +1457,9 @@ class Laps(pd.DataFrame):
         - **Time** (pandas.Timedelta): Session time when the lap time was set (end of lap)
         - **Driver** (string): Three letter driver identifier
         - **DriverNumber** (str): Driver number
-        - **LapTime** (pandas.Timedelta): Recorded lap time
+        - **LapTime** (pandas.Timedelta): Recorded lap time.
+          Officially deleted lap times will *not* be deleted here.
+          Deleting laps is currently not supported.
         - **LapNumber** (int): Recorded lap number
         - **Stint** (int): Stint number
         - **PitOutTime** (pandas.Timedelta): Session time when car exited the pit
@@ -1523,7 +1566,15 @@ class Laps(pd.DataFrame):
         """
         pos_data = self.get_pos_data(pad=1, pad_side='both')
         car_data = self.get_car_data(pad=1, pad_side='both')
-        car_data = car_data.add_distance().add_relative_distance().add_driver_ahead()
+
+        # calculate driver ahead from from data without padding to
+        # prevent out of bounds errors
+        drv_ahead = car_data.iloc[1:-1].add_driver_ahead() \
+            .loc[:, ('DriverAhead', 'DistanceToDriverAhead',
+                     'Date', 'Time', 'SessionTime')]
+
+        car_data = car_data.add_distance().add_relative_distance()
+        car_data = car_data.merge_channels(drv_ahead)
         merged = pos_data.merge_channels(car_data)
         return merged.slice_by_lap(self, interpolate_edges=True)
 
@@ -1570,12 +1621,71 @@ class Laps(pd.DataFrame):
             raise ValueError("Cannot slice telemetry because self contains no driver number!")
         if len(drv_num) > 1:
             raise ValueError("Cannot slice telemetry because self contains Laps of multiple drivers!")
-        pos_data = self.session.pos_data[self['DriverNumber']].slice_by_lap(self, **kwargs).reset_index(drop=True)
+        drv_num = drv_num[0]
+        pos_data = self.session.pos_data[drv_num].slice_by_lap(self, **kwargs).reset_index(drop=True)
         return pos_data
 
+    def get_weather_data(self):
+        """Return weather data for each lap in self.
+
+        Weather data is updated once per minute. This means that there are
+        usually one or two data points per lap. This function will always
+        return only one data point per lap:
+
+           - The first value within the duration of a lap
+
+        or
+
+            - the last known value before the end of the lap if there are
+              no values within the duration of a lap
+
+        See :func:`fastf1.api.weather_data` for available data
+        channels.
+
+        If you wish to have more control over the data, you can access the
+        weather data directly in :attr:`Session.weather_data`.
+
+        Returns:
+            pandas.DataFrame
+
+        Example::
+
+            >>> weather_data = laps.get_weather_data()
+            >>> print(weather_data)
+                                 Time AirTemp Humidity  ... TrackTemp WindDirection WindSpeed
+            16 0 days 00:16:20.305000    21.9     55.8  ...      35.5           119       0.2
+            17 0 days 00:17:20.301000    22.0     54.6  ...      35.9           113       0.1
+            ..                    ...     ...      ...  ...       ...           ...       ...
+            75 0 days 01:15:20.525000    22.3     53.5  ...      36.9            76       0.6
+            76 0 days 01:16:20.534000    22.2     53.1  ...      37.1            81       0.6
+            [453 rows x 25 columns]
+
+        Joining weather data with lap timing data::
+
+            >>> # prepare the data for joining
+            >>> laps = laps.reset_index(drop=True)
+            >>> weather_data = weather_data.reset_index(drop=True)
+
+            >>> # exclude the 'Time' column from weather data when joining
+            >>> joined = pandas.concat([laps, weather_data.loc[:, ~(weather_data.columns == 'Time')]], axis=1)
+            >>> print(joined)
+                                  Time DriverNumber  ... WindDirection  WindSpeed
+            0   0 days 00:16:58.221000            3  ...           119        0.2
+            1   0 days 00:19:32.354000            3  ...           113        0.1
+            ..                     ...          ...  ...           ...        ...
+            451 0 days 01:16:01.722000           99  ...            76        0.6
+            452 0 days 01:17:35.072000           99  ...            81        0.6
+            [453 rows x 32 columns]
+        """
+        wd = [lap.get_weather_data() for _, lap in self.iterrows()]
+        if wd:
+            return pd.concat(wd, axis=1).T
+        else:
+            return pd.DataFrame(columns=self.session.weather_data.columns)
+
     def pick_driver(self, identifier):
-        """Select and return all laps of a specific driver in self based on the driver's three letters identifier or
-        based on the driver number ::
+        """Return all laps of a specific driver in self based on the driver's
+        three letters identifier or based on the driver number ::
 
             perez_laps = ff1.pick_driver('PER')
             bottas_laps = ff1.pick_driver(77)
@@ -1594,8 +1704,10 @@ class Laps(pd.DataFrame):
             return self[self['Driver'] == identifier]
 
     def pick_drivers(self, identifiers):
-        """Select and return all laps of the specified drivers in self based on the drivers' three letters identifier
-        or based on the driver number. This is the same as :meth:`Laps.pick_driver` but for multiple drivers at once. ::
+        """Return all laps of the specified drivers in self based on the
+        drivers' three letters identifier or based on the driver number. This
+        is the same as :meth:`Laps.pick_driver` but for multiple drivers
+        at once. ::
 
             some_drivers_laps = ff1.pick_drivers([5, 'BOT', 7])
 
@@ -1612,7 +1724,8 @@ class Laps(pd.DataFrame):
         return self[(drv.isin(names) | num.isin(numbers))]
 
     def pick_team(self, name):
-        """Select and return all laps of a specific team in self based on the team's name ::
+        """Return all laps of a specific team in self based on the
+        team's name ::
 
             mercedes = ff1.pick_team('Mercedes')
             alfa_romeo = ff1.pick_team('Alfa Romeo')
@@ -1628,8 +1741,9 @@ class Laps(pd.DataFrame):
         return self[self['Team'] == name]
 
     def pick_teams(self, names):
-        """Select and return all laps of the specified teams in self based on the teams' name.
-        This is the same as :meth:`Laps.pick_team` but for multiple teams at once. ::
+        """Return all laps of the specified teams in self based on the teams'
+        name. This is the same as :meth:`Laps.pick_team` but for multiple
+        teams at once. ::
 
             some_drivers_laps = ff1.pick_teams(['Mercedes', 'Williams'])
 
@@ -1642,7 +1756,14 @@ class Laps(pd.DataFrame):
         return self[self['Team'].isin(names)]
 
     def pick_fastest(self):
-        """Select and return the lap with the fastest lap time.
+        """Return the lap with the fastest lap time.
+
+        .. note:: Officially deleted laps (e.g. driver was outside of
+          the track limits) will still show up as normal laps here.
+          The fastest lap returned by this method can therefore be one
+          that was officially deleted after it was set. You need to manually
+          ensure that this is not the case by comparing lap times with a
+          session's official results or some other source of data.
 
         Returns:
             instance of :class:`Lap`
@@ -1655,8 +1776,9 @@ class Laps(pd.DataFrame):
         return lap
 
     def pick_quicklaps(self, threshold=None):
-        """Select and return laps with `LapTime` faster than a certain limit.
-        By default the threshold is 107% of the best `LapTime` of all laps in self.
+        """Return all laps with `LapTime` faster than a certain limit. By
+        default the threshold is 107% of the best `LapTime` of all laps
+        in self.
 
         Args:
             threshold (optional, float): custom threshold coefficent
@@ -1672,7 +1794,7 @@ class Laps(pd.DataFrame):
         return self[self['LapTime'] < time_threshold]
 
     def pick_tyre(self, compound):
-        """Select and return all laps in self which were done on a specific compound.
+        """Return all laps in self which were done on a specific compound.
 
         Args:
             compound (string): may be "SOFT", "MEDIUM", "HARD", "INTERMEDIATE" or "WET"
@@ -1683,7 +1805,7 @@ class Laps(pd.DataFrame):
         return self[self['Compound'] == compound]
 
     def pick_track_status(self, status, how='equals'):
-        """Select and return laps set under a specific track status.
+        """Return all laps set under a specific track status.
 
         Args:
             status (str): The track status as a string, e.g. '1'
@@ -1701,7 +1823,7 @@ class Laps(pd.DataFrame):
             raise ValueError(f"Invalid value '{how}' for kwarg 'how'")
 
     def pick_wo_box(self):
-        """Select laps which are NOT in laps or out laps.
+        """Return all laps which are NOT in laps or out laps.
 
         Returns:
             instance of :class:`Laps`
@@ -1709,7 +1831,8 @@ class Laps(pd.DataFrame):
         return self[pd.isnull(self['PitInTime']) & pd.isnull(self['PitOutTime'])]
 
     def pick_accurate(self):
-        """Select and return laps which pass the accuracy validation check (lap['IsAccurate'] is True).
+        """Return all laps which pass the accuracy validation check
+        (lap['IsAccurate'] is True).
 
         Returns:
             instance of :class:`Laps`
@@ -1728,7 +1851,7 @@ class Laps(pd.DataFrame):
                iterator only yields laps for which this is true. If require is left empty, the iterator will yield
                all laps.
         Yields:
-            instance of :class:`Lap`
+            (index, instance of :class:`Lap`)
         """
         for index, lap in self.iterrows():
             if require:
@@ -1790,7 +1913,15 @@ class Lap(pd.Series):
         """
         pos_data = self.get_pos_data(pad=1, pad_side='both')
         car_data = self.get_car_data(pad=1, pad_side='both')
-        car_data = car_data.add_distance().add_relative_distance().add_driver_ahead()
+
+        # calculate driver ahead from from data without padding to
+        # prevent out of bounds errors
+        drv_ahead = car_data.iloc[1:-1].add_driver_ahead() \
+            .loc[:, ('DriverAhead', 'DistanceToDriverAhead',
+                     'Date', 'Time', 'SessionTime')]
+
+        car_data = car_data.add_distance().add_relative_distance()
+        car_data = car_data.merge_channels(drv_ahead)
         merged = pos_data.merge_channels(car_data)
         return merged.slice_by_lap(self, interpolate_edges=True)
 
@@ -1825,6 +1956,61 @@ class Lap(pd.Series):
         pos_data = self.session.pos_data[self['DriverNumber']].slice_by_lap(self, **kwargs).reset_index(drop=True)
         return pos_data
 
+    def get_weather_data(self):
+        """Return weather data for this lap.
+
+        Weather data is updated once per minute. This means that there are
+        usually one or two data points per lap. This function will always
+        return only one data point:
+
+            - The first value within the duration of a lap
+
+        or
+
+            - the last known value before the end of the lap if there are
+              no values within the duration of a lap
+
+        See :func:`fastf1.api.weather_data` for available data
+        channels.
+
+        If you wish to have more control over the data, you can access the
+        weather data directly in :attr:`Session.weather_data`.
+
+        Returns:
+            pandas.Series
+
+        Example::
+
+            >>> lap = laps.pick_fastest()
+            >>> lap['LapStartTime']
+            Timedelta('0 days 01:10:29.730000')
+            >>> lap.get_weather_data()
+            Time             0 days 01:11:20.525000
+            AirTemp                            22.1
+            Humidity                           53.5
+            Pressure                         1003.5
+            Rainfall                           True
+            TrackTemp                          36.9
+            WindDirection                        76
+            WindSpeed                           0.6
+            Name: 71, dtype: object
+        """
+        # get first value within the duration of the lap
+        mask = ((self.session.weather_data['Time'] >= self['LapStartTime']) &
+                (self.session.weather_data['Time'] <= self['Time']))
+        samples = self.session.weather_data[mask]
+        if not samples.empty:
+            return samples.iloc[0]
+
+        # fallback: get last value before the lap ended
+        mask = self.session.weather_data['Time'] <= self['Time']
+        samples = self.session.weather_data[mask]
+        if not samples.empty:
+            return samples.iloc[-1]
+
+        # no data: return an empty Series with the correct index names
+        return pd.Series(index=self.session.weather_data.columns)
+
 
 class Driver:
     """Driver class that provides some information on drivers and their finishing results.
