@@ -21,19 +21,20 @@ def delta_time(reference_lap, compare_lap):
         different way for verification.
 
     Here is an example that compares the quickest laps of Leclerc and
-    Hamilton from Bahrain 2021 Qualifying::
+    Hamilton from Bahrain 2021 Qualifying:
+
+    .. plot::
+        :include-source:
 
         import fastf1 as ff1
         from fastf1 import plotting
         from fastf1 import utils
-        from fastf1.livetiming.data import LiveTimingData
         from matplotlib import pyplot as plt
 
         plotting.setup_mpl()
 
-        livedata = LiveTimingData('recording.txt')
-        quali = ff1.get_session(2021, 'Bahrain', 'Q')
-        laps = quali.load_laps(with_telemetry=True, livedata=livedata)
+        quali = ff1.get_session(2021, 'Emilia Romagna', 'Q')
+        laps = quali.load_laps(with_telemetry=True)
         lec = laps.pick_driver('LEC').pick_fastest()
         ham = laps.pick_driver('HAM').pick_fastest()
 
@@ -49,12 +50,10 @@ def delta_time(reference_lap, compare_lap):
                 color=plotting.TEAM_COLORS[lec['Team']])
 
         twin = ax.twinx()
-        twin.plot(ref_tel['Distance'], delta_time, '--', color=plotting.TEAM_COLORS[lec['Team']])
+        twin.plot(ref_tel['Distance'], delta_time, '--', color='white')
+        twin.set_ylabel("<-- Lec ahead | Ham ahead -->")
         plt.show()
 
-    .. image:: _static/delta_time.svg
-        :target: _static/delta_time.svg
-
     Args:
         reference_lap (pd.Series): The lap taken as reference
         compare_lap (pd.Series): The lap to compare
